#!/usr/bin/env node
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
// import necessary files
import { writeFileSync, readdirSync, readFileSync, existsSync, copyFileSync, mkdirSync, renameSync, chmodSync, } from 'fs';
import * as os from 'os';
import * as rmdir from 'rimraf';
import * as forge from 'node-forge';
import keypair from 'keypair';
import yargs from 'yargs';
import { hideBin } from 'yargs/helpers';
// Get path for work
export var HOME_PATH = "".concat(os.homedir(), "/");
export var SSH_PATH = "".concat(HOME_PATH, ".ssh/");
export var SWITCH_FILE = "".concat(SSH_PATH, ".sshwitch");
// get buffer to string
export var bufferToString = function (buffer) {
    if (buffer === void 0) { buffer = ''; }
    return buffer.toString();
};
// Get the all directory aka keys
export var getDirectories = function (source) {
    var _a, _b;
    return (_b = (_a = readdirSync(source, {
        withFileTypes: true,
    })) === null || _a === void 0 ? void 0 : _a.filter(function (dirent) { return dirent.isDirectory(); })) === null || _b === void 0 ? void 0 : _b.map(function (dirent) { return dirent.name; });
};
// Get current key
export var getCurrent = function () {
    try {
        return readFileSync(SWITCH_FILE).toString();
    }
    catch (e) {
        writeFileSync(SWITCH_FILE, '');
        return bufferToString(readFileSync(SWITCH_FILE));
    }
};
// get all ssh keys
export var getList = function () {
    var _a;
    (_a = getDirectories(SSH_PATH)) === null || _a === void 0 ? void 0 : _a.map(function (item) { return console.log(item); });
    return getDirectories(SSH_PATH);
};
// Switch ssh key
export var switchKey = function (keyName) {
    try {
        if (existsSync("".concat(SSH_PATH + keyName, "/id_rsa")) &&
            existsSync("".concat(SSH_PATH + keyName, "/id_rsa.pub"))) {
            try {
                copyFileSync("".concat(SSH_PATH + keyName, "/id_rsa"), "".concat(SSH_PATH, "id_rsa"));
                copyFileSync("".concat(SSH_PATH + keyName, "/id_rsa.pub"), "".concat(SSH_PATH, "id_rsa.pub"));
                writeFileSync(SWITCH_FILE, keyName);
                chmodSync("".concat(SSH_PATH, "/id_rsa.pub"), 384);
                chmodSync("".concat(SSH_PATH, "/id_rsa"), 384);
                return console.log("Changed key pair to: ".concat(keyName));
            }
            catch (err) {
                console.log("Could not copy, check if you have permission to write on ".concat(SSH_PATH));
                return console.error(err);
            }
        }
        else {
            console.log("Could not read key pair in ".concat(SSH_PATH + keyName));
            console.log("Check if key pair exists in ".concat(SSH_PATH + keyName, "\n\n"));
            console.log("If not you can create a new one with: \"sshmulti -n ".concat(keyName, "\""));
            return console.log("Or backup the current key pair in ".concat(SSH_PATH, " with: \"sshmulti -b ").concat(keyName, "\""));
        }
    }
    catch (err) {
        return console.error(err);
    }
};
// Register new ssh key
export var newKey = function (newName) {
    if (existsSync(SSH_PATH + newName)) {
        return console.log("".concat(SSH_PATH + newName, " already exists, skipping"));
    }
    console.log("Creating key pair in ".concat(SSH_PATH + newName));
    try {
        mkdirSync(SSH_PATH + newName);
        var pair = keypair();
        var publicKey = forge.pki.publicKeyFromPem(pair.public);
        var privateKey = forge.pki.privateKeyFromPem(pair.private);
        var ssh = forge.ssh.publicKeyToOpenSSH(publicKey);
        var sshPrivate = forge.ssh.privateKeyToOpenSSH(privateKey);
        writeFileSync("".concat(SSH_PATH + newName, "/id_rsa.pub"), ssh);
        writeFileSync("".concat(SSH_PATH + newName, "/id_rsa"), sshPrivate);
        chmodSync("".concat(SSH_PATH + newName, "/id_rsa.pub"), 384);
        chmodSync("".concat(SSH_PATH + newName, "/id_rsa"), 384);
        copyFileSync("".concat(SSH_PATH + newName, "/id_rsa"), "".concat(SSH_PATH, "id_rsa"));
        copyFileSync("".concat(SSH_PATH + newName, "/id_rsa.pub"), "".concat(SSH_PATH, "id_rsa.pub"));
        writeFileSync(SWITCH_FILE, newName);
        return console.log("\n public key: \n\n ".concat(ssh, "\n private key: \n\n ").concat(sshPrivate));
    }
    catch (err) {
        console.log("Could not create key pair, check if you have permission to write on ".concat(SSH_PATH + newName));
        return console.log(err);
    }
};
// backup ssh keys
export var backupKey = function (oldName) {
    if (existsSync(SSH_PATH + oldName)) {
        return console.log("".concat(SSH_PATH + oldName, " already exists, skipping"));
    }
    console.log("Copying current key pair in ".concat(SSH_PATH, " to ").concat(SSH_PATH + oldName));
    try {
        mkdirSync(SSH_PATH + oldName);
        copyFileSync("".concat(SSH_PATH, "id_rsa"), "".concat(SSH_PATH + oldName, "/id_rsa"));
        copyFileSync("".concat(SSH_PATH, "id_rsa.pub"), "".concat(SSH_PATH + oldName, "/id_rsa.pub"));
        return console.log("Backup to current key pair in ".concat(SSH_PATH, " to ").concat(SSH_PATH + oldName));
    }
    catch (err) {
        console.log("Could not backup key pair, check if you have permission to write on ".concat(SSH_PATH + oldName));
        return console.log(err);
    }
};
// Rename ssh key
export var renameKey = function (oldName, newName) {
    if (!existsSync(SSH_PATH + oldName)) {
        return console.log("".concat(SSH_PATH + oldName, " doesn't exists, cannot rename"));
    }
    if (existsSync(SSH_PATH + newName)) {
        return console.log("".concat(SSH_PATH + newName, " already exists, cannot rename"));
    }
    try {
        renameSync(SSH_PATH + oldName, SSH_PATH + newName);
        if (getCurrent() === oldName) {
            writeFileSync(SWITCH_FILE, newName);
            switchKey(newName);
        }
        return console.log("Renamed ".concat(oldName, " key to ").concat(newName));
    }
    catch (err) {
        console.log('Could not rename key pair');
        return console.log(err);
    }
};
// delete ssh key
export var deleteKey = function (deletedName) {
    if (!existsSync(SSH_PATH + deletedName)) {
        return console.log("".concat(SSH_PATH + deletedName, " doesn't exists, cannot delete"));
    }
    try {
        rmdir.sync(SSH_PATH + deletedName);
        return console.log("Deleted ".concat(deletedName, " sshkey."));
    }
    catch (e) {
        return console.log(e);
    }
};
// Using argv for taking flag and input
export var parsedArg = yargs(hideBin(process.argv))
    .usage("\n    Switch and manage key pairs in ".concat(SSH_PATH, "\n    Default usage: sshmulti key_pair_name\n    Advanced usage: sshmulti [option]\n    -c, --current                    Get name of current key pair\n    -n, --new                        New key pair name\n    -b, --backup                     Backup key pair in ").concat(SSH_PATH, "\n    -r, --rename                     Rename a key pair\n    -l, --list                       Get list of key pairs in ").concat(SSH_PATH, "\n    -d, --delete DELETE              Delete key pair\n    "))
    .help('help')
    .alias('help', 'h')
    .version('version', process.env.npm_package_version || '3.0.2')
    .alias('version', 'V')
    .options({
    backup: { type: 'boolean', alias: ['backup', 'b'], default: false },
    new: { type: 'boolean', alias: ['new', 'n'], default: false },
    rename: { type: 'boolean', alias: ['rename', 'r'], default: false },
    list: { type: 'boolean', alias: ['list', 'l'], default: false },
    current: { type: 'boolean', alias: ['current', 'c'], default: false },
    delete: { type: 'boolean', alias: ['delete', 'd'], default: false },
}).argv;
export var handleCommand = function (argv) {
    var isBackup = argv.backup, isCurrent = argv.current, isDelete = argv.delete, isList = argv.list, isRename = argv.rename, isNew = argv.new, name = argv._;
    if (isBackup && name[0].toString().length >= 1) {
        return backupKey(name[0].toString());
    }
    if (isRename && name.length === 2) {
        return renameKey(name[0].toString(), name[1].toString());
    }
    if (isNew && name.length === 1) {
        return newKey(name[0].toString());
    }
    if (isDelete && name.length === 1) {
        return deleteKey(name[0].toString());
    }
    if (isList) {
        return getList();
    }
    if (isCurrent || name.length === 0) {
        return console.log(getCurrent());
    }
    if (!isBackup &&
        !isCurrent &&
        !isDelete &&
        !isList &&
        !isRename &&
        !isNew &&
        name.length === 1) {
        return switchKey(name[0].toString());
    }
    return console.log('Please enter valid name. Type sshmulti --help for more.');
};
(function () { return __awaiter(void 0, void 0, void 0, function () {
    var argv;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, parsedArg];
            case 1:
                argv = _a.sent();
                handleCommand(argv);
                return [2 /*return*/];
        }
    });
}); })();
export default getDirectories;
